/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */


package org.enhydra.barracuda.discRack.data.person;

import java.io.*;
import java.math.*;
import java.util.*;
import java.sql.*;

import com.lutris.util.Config;
import com.lutris.util.KeywordValueTable;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.dods.builder.generator.dataobject.GenericDO;
import com.lutris.dods.builder.generator.query.*;


/**
 * Data core class, used to set, retrieve the PersonDO information.
 *
 * @version $Revision: 1.2 $
 * @author  ChristianC
 * @since   org.enhydra.barracuda.discRack
 */
 public class PersonDO extends com.lutris.dods.builder.generator.dataobject.GenericDO implements PersonDOI, java.io.Serializable {


    /**
     * static final data members name the table and columns for this DO.
     * By using these members with an instance of the QueryBuilder class,
     * an application can perform straight SQL queries while retaining
     * compile-time checking of table and column usage.
     *
     * Example:  List the Cities containing Persons named Bob:
     *
     *    Using straight SQL with QueryBuilder:
     *      Pro: code runs faster because you create fewer objects
     *      Con: code is less clear
     *
     *         Vector fields = new Vector();
     *         fields.addElement( AddressDO.City );
     *         QueryBuilder qb = new QueryBuilder( fields );
     *         qb.addWhere( PersonDO.FirstName, "Bob" );
     *         qb.addWhere( PersonDO.PrimaryKey, AddressDO.Person );
     *         RDBRow row;
     *         while ( null != ( row = qb.getNextRow() ) ) {
     *             String city = row.get( AddressDO.City ).getString();
     *         }
     *
     *    Using Query/DO classes:
     *      Pro: code is (often) clearer
     *      Con: code runs slower because you create more objects
     *
     *         PersonQuery pq = new PersonQuery();
     *         pq.setQueryFirstName( "Bob" );
     *         PersonDO[] bobs = pq.getDOArray();
     *         for ( int i = 0; i < bobs.length; i++ ) {
     *             AddressQuery aq = new AddressQuery();
     *             aq.setQueryPerson( bobs[i] );
     *             AddressDO addr = aq.getNextDO();
     *             String city = addr.getCity();
     *         }
     */
    static public final RDBTable  table = new RDBTable( "person" );

    /**
     * Return person as the name of the table in the database
     * which contains PersonDO objects.
     * This method overrides CoreDO.getTableName()
     * and is used by CoreDO.executeUpdate() during error handling.
     *
     * @return the database table name
     * @see CoreDO
     * @author Jay Gunter
     */
    public String getTableName() {
	return "person";
    }


   /**
    * static final RDBColumn PrimaryKey for use with QueryBuilder.
    * See example above.
    */
    static public final RDBColumn PrimaryKey = new RDBColumn( table,
					      getPrimaryKeyName() );
    /* RDBColumns for PersonDO attributes are defined below. */



    /* Using a DO (and its Query class) to access a VIEW instead of a TABLE:
     *
     * A DO (and its Query class) can be used to access a VIEW
     * instead of a TABLE.  The Data Object is created as usual in DODS,
     * but the "create table" SQL command for that DO is not used.
     * Instead, you substitute a "create view" command to create a
     * virtual table in the database; this is often done to provide
     * convenient access to a collection of tables joined together.
     *
     * A VIEW usually does not return "oid" and "version" columns;
     * often (but now always) a VIEW is defined to return the "oid" column
     * for one of the tables joined together in the definition of the VIEW.
     *
     * If notUsingOId is true, PersonDO.createExisting(ResultSet)
     * will NOT invoke the GenericDO(ResultSet) constructor
     * so to avoid attempting to extract the "oid" and "version" columns
     * from the ResultSet.
     */
    
    static protected final boolean notUsingOId = false;
    

    /**
     * A DO class contains a reference to a DataStruct class.
     * This reference can be null (when the data for the DO
     * has not yet been retrieved from the database),
     * allowing a DO object to be a lightweight placeholder
     * until its data is needed.
     */
    private PersonDataStruct data = null;

    /**
     * isReadOnly()
     * Returns true if the data for this object has been marked read-only.
     */
    public boolean isReadOnly() {

	if ( null == data ) return false;
	return data.readOnly;

    }

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected PersonDO ( boolean is_view )
    throws ObjectIdException, DatabaseManagerException {
        super( is_view );
    }

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected PersonDO ()
    throws ObjectIdException, DatabaseManagerException {
        super( notUsingOId );
    }

    /**
     * isLoaded()
     * Returns true if the data for this objects has been retrieved
     * from the database.
     */
    public boolean isLoaded() {
	return ( null != data );
    }


    /**
     * loadData()
     * Load the fields for the DO from the database.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public void loadData()
    throws SQLException, ObjectIdException, DataObjectException
    {
	if ( null == data ) {
	    
	    data = new PersonDataStruct ();
	}

	ObjectId id = getOId();
	if ( null == id )
	    return;
	if ( ! isPersistent() )   // DO from createVirgin
	    return;

	// DO from createExisting.  Complain if no record in database.
	PersonQuery query = new PersonQuery ();
	query.setQueryOId( id );
	query.requireUniqueInstance();
	PersonDO obj;
	try {
	    obj = query.getNextDO();
	    if ( null == obj )
		throw new DataObjectException(
		    "PersonDO DO not found for id=" + id );
	    makeIdentical(obj);
	    setVersion(    obj.getVersion() );
	    setNewVersion( obj.getVersion() );

	} catch ( NonUniqueQueryException e ) {
	    throw new ObjectIdException( "Duplicate ObjectId" );
	}

    }



    /**
     * Load the actual DO data if necessary.
     * Called by get/set methods.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    private void checkLoad()
    throws DataObjectException {
	if ( null == data )
	    try {
		loadData();
	    } catch ( Exception e ) {
		throw new DataObjectException(
		    "Unable to load data for PersonDO id=" + getOId() +
		    ", error = " + e.getMessage() );
	    }
    }


    /**
     * Protected constructor used by createExisting(ObjectId) above.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   Should never see this exception since GenericDO.ctor(ObjectId)
     *   never accesses the database.
     */
    protected PersonDO( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	super( id );
    }




    /**
     * createVirgin()
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static PersonDO createVirgin()
    throws DatabaseManagerException, ObjectIdException {
	return new PersonDO ();
    }


    /**
     * createExisting( BigDecimal )
     *
     * Factory method creates a PersonDO object by searching for it
     * in the database using the passed BigDecimal value as the primary key.
     *
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( "bbb", 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static PersonDO createExisting( BigDecimal bd )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	if ( null == bd )
	    return null;
	return createExisting( new ObjectId( bd ) );
    }

    /**
     * The createExisting method is used to create a <CODE>PersonDO</CODE>
     * from a string handle.
     */
    public static PersonDO createExisting( String handle ) {
	PersonDO ret = null;
        try {
            BigDecimal bd = new BigDecimal( handle );
	    ret = createExisting( bd );
        } catch ( Exception e ) {
        }
	return ret;
    }

    /**
     * createExisting( ObjectId )
     *
     * Factory method creates a PersonDO object by searching for it
     * in the database using the passed ObjectID value as the primary key.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	if ( null == id )
	    return null;
	PersonDO ret = null;
	ret = new PersonDO( id );
	ret.setPersistent( true );  // mark DO as persistent (preexisting)
	if ( ! true ) // If not lazy-loading, fetch DO data now.
	    ret.loadData();
	// unset the GenericDO.dirty flag.
	ret.markClean();
	return ret;
    }


    /**
     * createExisting( ResultSet )
     *
     * Factory method used to create an instance of this class to
     * represent a Data Object already existing in the database.
     *
     * @param rs The ResultSet returned by the Query class for
     * an existing Data Object stored in the database.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == rs )
	    return null;
	PersonDO ret = null;
	if ( notUsingOId ) {
	    ret = new PersonDO ();
	    ret.initFromResultSet( rs );
		ret.setPersistent( true );  // mark DO as persistent (preexisting)	//csc_11901
	} else {
	    ret = new PersonDO ( rs );
	}
	return ret;
    }


    /**
     * createExisting( RDBRow )
     *
     * Factory method creates a PersonDO object by searching for it
     * in the database using the PersonDO.PrimaryKey value
     * in the passed RDBRow.
     *
     * @param RDBRow A row returned by QueryBuilder.getNextRow().
     *
     * @exception DataObjectException
     *   If the RDBRow does not contain a PersonDO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( RDBRow row )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == row )
	    return null;
        RDBColumnValue pk = null;
        try {
	    pk = row.get( PersonDO.PrimaryKey );
	    return createExisting( pk );
        } catch ( Exception e ) {
	    throw new DataObjectException(
		"Cannot create PersonDO, row does not " +
		"contain PersonDO primary key." );
        }
    }

    /**
     * createExisting( RDBColumnValue )
     *
     * Factory method creates a PersonDO object by searching for it
     * in the database using the passed PersonDO.PrimaryKey.
     *
     * @param RDBColumnValue a PrimaryKey column value from a row
     * that was returned by QueryBuilder.getNextRow().
     *
     * @exception DataObjectException
     *   If the RDBColumnValue does not contain a PersonDO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( RDBColumnValue pk )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == pk )
	    return null;
	if ( ! pk.equals( PersonDO.PrimaryKey ) )
	    throw new DataObjectException(
		"Cannot create PersonDO, " +
		"RDBColumnValue is not PersonDO.PrimaryKey." );
	BigDecimal bd = null;
        try {
	    bd = pk.getBigDecimal();
        } catch ( Exception e ) {
	    throw new DataObjectException(
		"Cannot create PersonDO, bad primary key." );
        }
	if ( null == bd )
            return null;
	return createExisting( bd );
    }



    /**
     * createCopy()
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param data The data struct to copy values from.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static PersonDO createCopy( PersonDataStruct data )
    throws DatabaseManagerException, ObjectIdException {
	PersonDO ret = new PersonDO ();
	ret.data = ( PersonDataStruct ) data.duplicate();
	return ret;
    }

    /**
     * createCopy()
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param orig The original DO to copy.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static PersonDO createCopy( PersonDO orig )
    throws DatabaseManagerException, ObjectIdException {
        if ( null == orig ) return null;
	PersonDO ret = new PersonDO ();
	if ( null != orig.data ) {
	    ret.data = ( PersonDataStruct ) orig.data.duplicate();
	}
	return ret;
    }



    /**
     * reload()
     * Causes the DO to refresh itself from the database
     * the next time a set or get method is called.
     */
    public void reload() {
	data = null;
    }




    /**
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle
     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with Data Objects as options.
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>
     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     *
     * @return id of this DO as a string
     *   If an object id can't be allocated for this object.
     */
    public        String  getHandle()
    throws DatabaseManagerException {
	String ret = null;
	    if ( null == getOId() )
		   throw new DatabaseManagerException( "ID not set" );
	    ret = getOId().toString();
        return ret;
    }



   /**
     * hasMatchingHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle
     * @see getHandle
     */
    public        boolean hasMatchingHandle( String handle ) {
	boolean ret = false;
	    if ( null == getOId() )
		   ret = false;
	    else
		   ret = getOId().toString().equals( handle );
        return ret;
    }



    /**
     * makeIdentical()
     *
     * Assigns the DataStruct of an existing DO to this DO.
     * Does not duplicate data. Just assigns the reference.
     *
     * @param orig The original DO.
     *
     */
    protected void makeIdentical( PersonDO orig ) {
	super.makeIdentical(orig);
	data = orig.data;
    }

    /**
     * getVersion make public the protected method in CoreDO.
     *
     */
    public int getVersion() {
	return super.getVersion();
    }




////////////////////////// data member Login
   static public final int login_MaxLength = 32;

   /* static final RDBColumn Login for use with QueryBuilder.
    * See RDBColumn PrimaryKey at the top of this file for usage example.
    */
   static public final RDBColumn Login =
		new RDBColumn( table, "login", true );

   
   /**
    * Set login of the person
    *
    * @param login of the person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setLogin ( String login )
   throws DataObjectException {
   
      _setLogin ( login );
   
   }
   


   /**
    * _setLogin is a protected method that is called by
    * setLogin if login is not part of
    * a multicolumn foreign key.
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   protected void _setLogin ( String login )
   throws DataObjectException {
      
      checkLoad();
      
      data.login =  markNewValue(
	data.login, login , 0, login_MaxLength, false );
   }

   /**
    * Get login of the person
    *
    * @return login of the person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getLogin ()
   throws DataObjectException {
   
      
      checkLoad();
      
   
      return data.login;
   }


////////////////////////// data member Password
   static public final int password_MaxLength = 32;

   /* static final RDBColumn Password for use with QueryBuilder.
    * See RDBColumn PrimaryKey at the top of this file for usage example.
    */
   static public final RDBColumn Password =
		new RDBColumn( table, "password", true );

   
   /**
    * Set password of the person
    *
    * @param password of the person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPassword ( String password )
   throws DataObjectException {
   
      _setPassword ( password );
   
   }
   


   /**
    * _setPassword is a protected method that is called by
    * setPassword if password is not part of
    * a multicolumn foreign key.
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   protected void _setPassword ( String password )
   throws DataObjectException {
      
      checkLoad();
      
      data.password =  markNewValue(
	data.password, password , 0, password_MaxLength, false );
   }

   /**
    * Get password of the person
    *
    * @return password of the person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPassword ()
   throws DataObjectException {
   
      
      checkLoad();
      
   
      return data.password;
   }


////////////////////////// data member Firstname
   static public final int firstname_MaxLength = 32;

   /* static final RDBColumn Firstname for use with QueryBuilder.
    * See RDBColumn PrimaryKey at the top of this file for usage example.
    */
   static public final RDBColumn Firstname =
		new RDBColumn( table, "firstname", true );

   
   /**
    * Set firstname of the person
    *
    * @param firstname of the person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setFirstname ( String firstname )
   throws DataObjectException {
   
      _setFirstname ( firstname );
   
   }
   


   /**
    * _setFirstname is a protected method that is called by
    * setFirstname if firstname is not part of
    * a multicolumn foreign key.
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   protected void _setFirstname ( String firstname )
   throws DataObjectException {
      
      checkLoad();
      
      data.firstname =  markNewValue(
	data.firstname, firstname , 0, firstname_MaxLength, false );
   }

   /**
    * Get firstname of the person
    *
    * @return firstname of the person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getFirstname ()
   throws DataObjectException {
   
      
      checkLoad();
      
   
      return data.firstname;
   }


////////////////////////// data member Lastname
   static public final int lastname_MaxLength = 32;

   /* static final RDBColumn Lastname for use with QueryBuilder.
    * See RDBColumn PrimaryKey at the top of this file for usage example.
    */
   static public final RDBColumn Lastname =
		new RDBColumn( table, "lastname", true );

   
   /**
    * Set lastname of the person
    *
    * @param lastname of the person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setLastname ( String lastname )
   throws DataObjectException {
   
      _setLastname ( lastname );
   
   }
   


   /**
    * _setLastname is a protected method that is called by
    * setLastname if lastname is not part of
    * a multicolumn foreign key.
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   protected void _setLastname ( String lastname )
   throws DataObjectException {
      
      checkLoad();
      
      data.lastname =  markNewValue(
	data.lastname, lastname , 0, lastname_MaxLength, false );
   }

   /**
    * Get lastname of the person
    *
    * @return lastname of the person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getLastname ()
   throws DataObjectException {
   
      
      checkLoad();
      
   
      return data.lastname;
   }



    /**
     * logicalDbName is the value set by setLogicalDBName()
     * and retrieved by getLogicalDBName().
     */
    static private String logicalDbName = null;

    /**
     * setLogicalDBName sets the logical database name that will be used
     * to create DBTransaction and DBQuery objects used by
     * PersonDO and the corresponding Query class.
     *
     * @param logicalDbNameInConfFile the logical database specified in the
     * application's .conf file.
     */
    static public void setLogicalDBName( String logicalDbNameInConfFile ) {
	if ( null != logicalDbNameInConfFile &&
	     0    != logicalDbNameInConfFile.length() )
	     logicalDbName = logicalDbNameInConfFile;
    }

    /**
     * getLogicalDBName retrieves the logical database name
     * set by setLogicalDBName().
     * @return the logical database name that was set using setLogicalDBName()
     */
    static public String getLogicalDBName() {
	return logicalDbName;
    }


    /**
     * createConnection() returns a Connection.
     * This method uses the logical database name set using setLogicalDBName().
     *
     * The PersonQuery.runQuery() method uses this method.
     *
     * @return a Connection object for use by the PersonQuery class.
     */
    static public Connection createConnection()
    throws DatabaseManagerException, SQLException {
        try {
	    DBConnection dbconn = null;
	    if ( null == logicalDbName )
		dbconn = Enhydra.getDatabaseManager().allocateConnection();
	    else
		dbconn = Enhydra.getDatabaseManager().allocateConnection(
								logicalDbName );
	    if ( null == dbconn )
	    	throw new DatabaseManagerException(
			"DatabaseManager returned null DBConnection." );
	    return dbconn.getConnection();
        } catch ( DatabaseManagerException e ) {
            String err = "";
	    if ( null != logicalDbName )
            	err =
		"ERROR: Could not allocate a DBConnection.  " +
		"PersonDO.getLogicalDBName()='" + logicalDbName + "'.  "+
		"The application .conf file must list this name in " +
		"DatabaseManager.Databases[], and there must be " +
		"DatabaseManager.DB." + logicalDbName + " settings.";
            throw new DatabaseManagerException( err, e );
        }
    }

    /**
     * createTransaction() creates a new DBTransaction.
     * This method uses the logical database name set using setLogicalDBName().
     *
     * If setLogicalDBName() was used to set the logical database name
     * to something other than the value of DatabaseManager.DefaultDatabase
     * in the application's .conf file, then any DBTransaction passed to
     * save(DBTransaction) or delete(DBTransaction) should be created using
     * PersonDO.createTransaction().
     *
     * The PersonDO save() and delete() methods use this method.
     *
     * @return a DBTransaction object for use with the PersonDO class.
     */
    static public DBTransaction createTransaction()
    throws DatabaseManagerException, SQLException {
        try {
	    if ( null == logicalDbName )
		return Enhydra.getDatabaseManager().createTransaction();
	    else
		return Enhydra.getDatabaseManager().createTransaction(
								logicalDbName );
        } catch ( DatabaseManagerException e ) {
            String err = "";
	    if ( null != logicalDbName )
            	err =
		"ERROR: Could not create a DBTransaction.  " +
		"PersonDO.getLogicalDBName()='" + logicalDbName + "'.  "+
		"The application .conf file must list this name in " +
		"DatabaseManager.Databases[], and there must be " +
		"DatabaseManager.DB." + logicalDbName + " settings.";
            throw new DatabaseManagerException( err, e );
        }
    }

    /**
     * createQuery() creates a new DBQuery.
     * This method uses the logical database name set using setLogicalDBName().
     *
     * If setLogicalDBName() was used to set the logical database name
     * to something other than the value of DatabaseManager.DefaultDatabase
     * in the application's .conf file, then any DBQuery object used to
     * access the 'person' table should be created using
     * PersonDO.createQuery().
     *
     * The Query class corresponding to PersonDO uses this method.
     *
     * @return a DBQuery object for use in accessing the 'person' table.
     */
    static public DBQuery createQuery()
    throws DatabaseManagerException, SQLException {
        try {
	    if ( null == logicalDbName )
		return Enhydra.getDatabaseManager().createQuery();
	    else
		return Enhydra.getDatabaseManager().createQuery( logicalDbName);
        } catch ( DatabaseManagerException e ) {
            String err = "";
	    if ( null != logicalDbName )
            	err =
		"ERROR: Could not create a DBQuery.  " +
		"PersonDO.getLogicalDBName()='" + logicalDbName + "'.  "+
		"The application .conf file must list this name in " +
		"DatabaseManager.Databases[], and there must be " +
		"DatabaseManager.DB." + logicalDbName + " settings.";
            throw new DatabaseManagerException( err, e );
        }
    }



    /**
     * Protected constructor.
     *
     * @param rs
     *   Result set from which to obtain product data.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected PersonDO(ResultSet rs)
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
    {
        super(rs);
	initFromResultSet( rs );
//	super.init( rs );
    }

    /**
     * initFromResultSet initializes the data members from person.
     * This code was separated from the ResultSet constructor
     * so that createExisting(ResultSet) could handle VIEWs.
     */
    private void initFromResultSet( ResultSet rs )
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
    {
	// Constructing a DO from a ResultSet means we definitely need the
	// DataStruct ready for the setXxx methods invoked below.
	data = new PersonDataStruct ();

	// writeMemberStuff uses the ResultSetExtraction.template
	// to build up the value for this tag:
	// the value is a series of calls to the DO set methods.


	
	_setLogin( 
	    
		rs.getString( 
			"login"  )
	    
	);
	


	
	_setPassword( 
	    
		rs.getString( 
			"password"  )
	    
	);
	


	
	_setFirstname( 
	    
		rs.getString( 
			"firstname"  )
	    
	);
	


	
	_setLastname( 
	    
		rs.getString( 
			"lastname"  )
	    
	);
	



        markClean();
    }



    

    

    private int[] param = null;

    public PreparedStatement getInsertStatement( DBConnection conn)
    throws SQLException {
        return getInsertStatement( conn.getConnection() );
    }

    /**
     * Prepares the statement used to insert this object
     * into the database.
     * @param conn the database connection.
     * @return the insert statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getInsertStatement( Connection conn)
    throws SQLException {
	/*
	 * It would probably be better to have CoreDO implement
	 * 	void addToCache(CoreDO DO) {}
	 * and have each DO that has caching enabled override it as
	 *      void addToCache(CoreDO DO) { cache.put( DO.getOId(), DO ); }
	 * and change CoreDO to invoke addToCache()
	 * when it invokes getInsertStatement().
	 */

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement(
	    "insert into person ( login, password, firstname, lastname, " + getOIdColumnName() + ", " + getVersionColumnName() + " ) values ( ?, ?, ?, ?, ?, ? )" );

	param = new int[1]; param[0] = 1;
	// writeMemberStuff uses the JDBCsetCalls.template
	// to build up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined in GenericDO.
	try {
	    setPrepStmtParam_String( stmt, param,
		getLogin() );
	    setPrepStmtParam_String( stmt, param,
		getPassword() );
	    setPrepStmtParam_String( stmt, param,
		getFirstname() );
	    setPrepStmtParam_String( stmt, param,
		getLastname() );



	    /* The ObjecId/Version columns form the primary key.  */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getNewVersion() );


	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }


    public PreparedStatement getUpdateStatement( DBConnection conn)
    throws SQLException {
        return getUpdateStatement( conn.getConnection() );
    }

    /**
     * Prepares the statement used to update this object
     * in the database.
     * @param conn the database connection
     * @return the update statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getUpdateStatement( Connection conn
    ) throws SQLException {

        PreparedStatement stmt = conn.prepareStatement(

	    "Update person set " +
	    getVersionColumnName() + " = ?, login = ?, password = ?, firstname = ?, lastname = ? " +
	    "where " + getOIdColumnName() + " = ? and " + getVersionColumnName() + " = ?"

	);

	param = new int[1]; param[0] = 1;
	try {

	    setPrepStmtParam_int( stmt, param, getNewVersion() );

	    setPrepStmtParam_String( stmt, param,
		getLogin() );
	    setPrepStmtParam_String( stmt, param,
		getPassword() );
	    setPrepStmtParam_String( stmt, param,
		getFirstname() );
	    setPrepStmtParam_String( stmt, param,
		getLastname() );



	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    public PreparedStatement getDeleteStatement( DBConnection conn)
    throws SQLException {
        return getDeleteStatement( conn.getConnection() );
    }

    /**
     * Prepares the statement used to delete this object
     * from the database.
     * @param conn the database connection
     * @return the delete statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getDeleteStatement( Connection conn
    ) throws SQLException {

        String sql =

            "delete from person \n" +
            "where " + getOIdColumnName() + " = ?";

        PreparedStatement stmt = conn.prepareStatement(sql);

        stmt.setBigDecimal(1, getOId().toBigDecimal());

        return stmt;
    }



    


    /*
     * toString - for debugging
     */
    public String toString(){
        return toString( 1 );
    }
    public String toString( int indentCount ){
        String indent = "";
        for ( int i = 0; i < indentCount; i++ ) {
            indent += ". ";
        }
        String str = indent + "PersonDO:";

        ObjectId oid = getOId();
        String id = "virgin";
        if ( null != oid )
            id = oid.toString();
        str += " OID=" + id + ",VERSION=" + getVersion();

        if ( null != data )
            str = str 
+ "\n" + indent + "login=" + data.login


+ "\n" + indent + "password=" + data.password


+ "\n" + indent + "firstname=" + data.firstname


+ "\n" + indent + "lastname=" + data.lastname

;
        return str + "\n" + indent + "SUPER=" + super.toString( indentCount );
        //return str;
    }



    /**
     * Get array of DiscDO objects that refer to this DO.
     *
     * @return array of DiscDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public org.enhydra.barracuda.discRack.data.disc.DiscDO[] getDiscDOArray () 
    throws DataObjectException, QueryException {
	org.enhydra.barracuda.discRack.data.disc.DiscDO[] ret = null;
	try {
	    org.enhydra.barracuda.discRack.data.disc.DiscQuery q = new org.enhydra.barracuda.discRack.data.disc.DiscQuery();

	    q.setQueryOwner( this );

	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.enhydra.barracuda.discRack.data.disc.DiscDO[ 0 ];
	}
	return ret;
    }

    /**
     * Get the single DiscDO object
     * that refers to this DO.
     *
     * @return DiscDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one DiscDO object was found.
     */
    public org.enhydra.barracuda.discRack.data.disc.DiscDO getDiscDO () 
    throws DataObjectException, QueryException, NonUniqueQueryException {
	org.enhydra.barracuda.discRack.data.disc.DiscQuery q = new org.enhydra.barracuda.discRack.data.disc.DiscQuery();

	    q.setQueryOwner( this );

	q.requireUniqueInstance();
	return q.getNextDO();
    }



    /**
     * Add (set & commit) a DiscDO object that refers to this DO.
     *
     * @param referrer DiscDO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addDiscDO( org.enhydra.barracuda.discRack.data.disc.DiscDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        addDiscDO( referrer, null );
    }

    /**
     * Add (set & commit) a DiscDO object that refers to this DO.
     *
     * @param referrer DiscDO to be set to point to this DO and committed.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addDiscDO( org.enhydra.barracuda.discRack.data.disc.DiscDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.setOwner( this );
        referrer.commit( tran );
    }

    /**
     * Remove (delete) a DiscDO object that refers to this DO.
     *
     * @param referrer DiscDO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeDiscDO( org.enhydra.barracuda.discRack.data.disc.DiscDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        removeDiscDO( referrer, null );
    }
 
    /**
     * Remove (delete) a DiscDO object that refers to this DO.
     *
     * @param referrer DiscDO to be deleted.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeDiscDO( org.enhydra.barracuda.discRack.data.disc.DiscDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	PersonDO referred = referrer.getOwner();
	String referredHandle = referred.getHandle();
	String mydoHandle = this.getHandle();
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran );
    }


 





    /**
     * A stub method for implementing pre-commit assertions
     * for this PersonDO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for writing to the database.
     */
    protected void okToCommit()
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-delete assertions
     * for this PersonDO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for deletion from the database.
     */
    protected void okToDelete()
    throws RefAssertionException { }


  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * @deprecated Use save() instead.
   */
  public void commit()
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    modifyDO( null, false );
  }

  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void save()
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    modifyDO( null, false );
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another DO
   * which references this DO.
   *
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * @deprecated Use save() instead.
   */
  public void commit(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    modifyDO( dbt, false );
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another DO
   * which references this DO.
   *
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void save(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    modifyDO( dbt, false );
  }

  /**
   * Deletes the DO from its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete()
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    modifyDO( null, true );
  }

  /**
   * Deletes the DO from its table.
   * The transaction is likely provided by the delete() method of another DO
   * which references this DO.
   *
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    modifyDO( dbt, true );
  }

  

  /**
   * Modifies the DO within its table.
   * Performs recursive commit/delete on referenced DOs;
   * all operations occur within a single transaction
   * to allow rollback in the event of error.
   * Only the creator of the transaction releases it.
   *
   * @param dbt The transaction object to use for this operation.
   * @param delete True if doing a delete, otherwise doing insert/update.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  protected void modifyDO( DBTransaction dbt, boolean delete )
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    if ( delete )
	okToDelete();
    else
	okToCommit();
    boolean ownTransaction = false;
    try {
      if ( null == dbt ) {
	//DatabaseManager dbm = Enhydra.getDatabaseManager();
        //dbt = dbm.createTransaction();      // create a transaction
        dbt = createTransaction();      // create a transaction
	ownTransaction = true;
      }
      if ( null == dbt )
	throw new DatabaseManagerException(
		"DatabaseManager.createTransaction returned null." );
      if ( delete ) {
	  // Code to perform cascading deletes is generated here
	  // if cascading deletes are not supported by the database.
	  	
	{
	    // perform cascading delete on referring table
	    org.enhydra.barracuda.discRack.data.disc.DiscDO[] a = getDiscDOArray();
	    for ( int i = 0; i < a.length; i++ ) {

		a[ i ].delete( dbt );

	    }
	}
	

	  // The following line keeps the compiler happy
	  // when the CASCADING_DELETES tag is empty.
          if ( false )
	      throw new QueryException("XXX");
      } else {
	  // commit referenced DOs.
	  
      }
      if ( false ) {
	  // This throw is here to keep the compiler happy
	  // in the case of a DO that does not refer to other DOs.
	  // In that case, the above delete/commit code blocks will be empty
	  // and throw nothing.
	  throw new DataObjectException( "foo" );
      }
      if ( delete ) {
          dbt.delete( this );
      } else {
	  if ( isLoaded() )
	      dbt.insert( this );   // dbt.insert() handles insertions and updates
      }
      if (ownTransaction) {
	  dbt.commit(); // commit the transaction
      }
    } catch (SQLException sqle) {
      StringBuffer message = new StringBuffer("Failed to insert/update DO: ");
      message.append(sqle.getMessage());

      // rollback, if necessary
      if (ownTransaction) {
        try {
          dbt.rollback();
        } catch (SQLException sqle2) {
          message.insert(0,"\n");
          message.insert(0,sqle2.getMessage());
          message.insert(0,"Rollback failed: ");
        }
      }
      throw new SQLException(message.toString());
    } finally {
      // release the transaction, if any
      if (ownTransaction) {
        dbt.release();
      }
    }
  }




    /**
     * Return the name of the database vendor (e.g. "Oracle")
     * for which this class was generated.
     */
    public static String getDbVendor() {
    	return "Standard";
    }

    /**
     * Return true if this class was generated with the -enhydra3 option.
     */
    public static boolean isEnhydra3() {
    	return isEnhydra3;
    }

    /**
     * Return true if this class is using the DriverManager to
     * obtain connections.  This can be true only if isEnhydra3 is false.
     */
    public static boolean isUsingDriverManager() {
    	return isUsingDriverManager;
    }

    public static final boolean isEnhydra3		= true;
    public static final boolean isUsingDriverManager	= false;

}
